import 'package:flavored_env/src/config.dart';
import 'package:flavored_env/src/utils.dart';
import 'package:yaml/yaml.dart';

/// Represents a constant to be used in both .env file
/// and a dart configuration file
class EnvVar {
  /// Constructor
  EnvVar({
    required this.name,
    this.value = '',
    this.info = '',
  });

  factory EnvVar.fromYamlMap(String name, YamlMap? yamlMap) {
    return EnvVar(
      name: name,
      info: yamlMap?[infoField] as String? ?? '',
      value: yamlMap?[valueField] ?? '',
    );
  }

  final String? info;
  final String name;
  final dynamic value;

  // dynamic get value => _value.runtimeType != String ? _value : "'$_value'";

  Type get valueType => value.runtimeType;

  String get toEnv {
    return '\n$kCommentEnv $info'
        '\n${camelCaseToUpperUnderscore(name)}=$value';
  }

  String get quotedValue => (valueType == String) ? "'$value'" : '$value';

  String get toDart {
    return '\n\t$kCommentDart $info'
        '\n$valueType get ${underscoreToCamelCase(name)} => $quotedValue;';
  }
}
